package com.lsf.dao;

import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.Writer;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Vector;

import oracle.sql.BLOB;
import oracle.sql.CLOB;

import org.apache.commons.io.IOUtils;

import com.lsf.db.Sql;
import com.lsf.support.Data;
import com.lsf.support.MultiData;
import com.lsf.util.CollectionUtility;
import com.lsf.util.CommonMap;

public abstract class AbstractDao {
	protected Connection conn;

	public AbstractDao(Connection conn) {
		this.conn = conn;
	}

	protected String getQuery(String id) {
		return Sql.getSql(id);
	}

	protected void setParams(PreparedStatement pstmt, HashMap map,
			String[] params) throws SQLException {
		for (int i = 1; i <= params.length; i++) {
			Object obj = map.get(params[i - 1]);
			if (obj == null)
				obj = "";
			setParam(pstmt, i, obj);
		}
	}

	private void setParam(PreparedStatement pstmt, int idx, Object obj)
			throws SQLException {
		String objType = obj.getClass().getName();
		if (Integer.class.getName().equals(objType))
			pstmt.setInt(idx, ((Integer) obj).intValue());
		else if (Double.class.getName().equals(objType))
			pstmt.setDouble(idx, ((Double) obj).doubleValue());
		else if (Float.class.getName().equals(objType))
			pstmt.setFloat(idx, ((Float) obj).floatValue());
		else if (Long.class.getName().equals(objType))
			pstmt.setLong(idx, ((Long) obj).longValue());
		else if (Short.class.getName().equals(objType))
			pstmt.setShort(idx, ((Short) obj).shortValue());
		else if (String.class.getName().equals(objType))
			pstmt.setString(idx, ((String) obj));
	}

	protected void close(ResultSet rs) {
		try {
			rs.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected void close(Statement stmt) {
		try {
			stmt.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected void close(PreparedStatement pstmt) {
		try {
			pstmt.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	protected void close(PreparedStatement pstmt, ResultSet rs) {
		close(rs);
		close(pstmt);
	}

	protected MultiData executeQueryList(String query, Data data,
			String[] params) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		MultiData result = null;
		try {
			pstmt = conn.prepareStatement(query,
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_UPDATABLE);
			this.setParams(pstmt, data, params);
			rs = pstmt.executeQuery();
			result = CollectionUtility.getMultiData(rs);
			printQuery(query, data, params);
		} catch (SQLException e) {
			printQuery(query, data, params, e);
			throw e;
		} finally {
			close(pstmt, rs);
		}
		return result;
	}

	/**
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @param page
	 *            1 base
	 * @param pageSize
	 * @return
	 * @throws SQLException
	 */
	protected MultiData executeQueryList(String query, Data data,
			String[] params, int page, int pageSize) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		MultiData result = null;
		try {
			pstmt = conn.prepareStatement(query,
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_UPDATABLE);
			this.setParams(pstmt, data, params);
			rs = pstmt.executeQuery();
			if (page == 0)
				page = 1;
			int start = (page - 1) * pageSize;
			result = CollectionUtility.getMultiData(rs, start, pageSize);
			printQuery(query, data, params);
		} catch (SQLException e) {
			printQuery(query, data, params, e);
			throw e;
		} finally {
			close(pstmt, rs);
		}
		return result;
	}

	/**
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @param page
	 *            1 base
	 * @param pageSize
	 * @return
	 * @throws SQLException
	 */
	protected MultiData executeQueryList(String query, Data data,
			String[] params, int page) throws SQLException {
		int pageSize = Integer.parseInt(CommonMap
				.getValue("/prj/system/pageSize"));
		return executeQueryList(query, data, params, page, pageSize);
	}

	protected Data executeQueryListWithHeader(String query, Data data,
			String[] params, int page, int pageSize) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Data result = new Data();
		Data info = new Data();
		MultiData mData = null;
		try {
			pstmt = conn.prepareStatement(query,
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_UPDATABLE);
			this.setParams(pstmt, data, params);
			rs = pstmt.executeQuery();
			int rowCount = 0;
			if (rs != null) {
				rs.last();
				rowCount = rs.getRow();
			}

			info.setInt("RowCount", rowCount);

			if (page == 0)
				page = 1;
			int start = (page - 1) * pageSize;
			mData = CollectionUtility.getMultiData(rs, start, pageSize);
			printQuery(query,data,params);
		} catch (SQLException e) {
			printQuery(query, data, params, e);
			throw e;
		} finally {
			close(pstmt, rs);
			result.put("ResultSet", mData);
			result.put("Info", info);
		}

		return result;
	}

	protected Data executeQuerySingle(String query, Data data, String[] params)
			throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Data result = null;
		try {
			pstmt = conn.prepareStatement(query,
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_UPDATABLE);
			this.setParams(pstmt, data, params);
			rs = pstmt.executeQuery();
			result = CollectionUtility.getData(rs);
			printQuery(query, data, params);
		} catch (SQLException e) {
			printQuery(query, data, params, e);
			throw e;
		} finally {
			close(pstmt, rs);
		}
		return result;
	}

	protected int executeUpdate(String query, Data data, String[] params)
			throws SQLException {
		PreparedStatement pstmt = null;
		int result = 0;
		try {
			pstmt = conn.prepareStatement(query);
			this.setParams(pstmt, data, params);
			result = pstmt.executeUpdate();
			printQuery(query, data, params);
		} catch (SQLException e) {
			printQuery(query, data, params, e);
			throw e;
		} finally {
			close(pstmt);
		}
		return result;
	}

	protected int executeUpdate(String query) throws SQLException {
		PreparedStatement pstmt = null;
		int result = 0;
		try {
			pstmt = conn.prepareStatement(query);
			result = pstmt.executeUpdate();
			printQuery(query, new Data(), new String[0]);
		} catch (SQLException e) {
			printQuery(query, new Data(), new String[0], e);
			throw e;
		} finally {
			close(pstmt);
		}
		return result;
	}

	/**
	 * CLOB ͸ Update Ѵ.
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @param colunmName
	 * @param content
	 * @throws Exception
	 */
	protected void executeUpdateClob(String query, Data data, String[] params,
			String colunmName, String content) throws Exception {
		PreparedStatement pstmt = null;
		ResultSet rs = null;

		try {
			pstmt = conn.prepareStatement(query);
			this.setParams(pstmt, data, params);
			printQuery(query, data, params);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				CLOB clob = (CLOB) rs.getClob(colunmName);
				Writer writer = clob.getCharacterOutputStream();
				StringReader reader = new StringReader(content);
				IOUtils.copy(reader, writer);
				reader.close();
				writer.close();
				printQuery(query, data, params);
			}
		} catch (SQLException e) {
			printQuery(query, new Data(), new String[0], e);
			throw e;
		} finally {
			try {
				rs.close();
			} catch (Exception e) {
			}
			try {
				pstmt.close();
			} catch (Exception e) {
			}
		}
	}

	/**
	 * BLOB ͸ Update Ѵ.
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @param colunmName
	 * @param is
	 * @throws Exception
	 */
	protected void executeUpdateBlob(String query, Data data, String[] params,
			String colunmName, InputStream is) throws Exception {
		PreparedStatement pstmt = null;
		ResultSet rs = null;

		try {
			pstmt = conn.prepareStatement(query);
			this.setParams(pstmt, data, params);
			printQuery(query, data, params);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				BLOB blob = (BLOB) rs.getBlob(colunmName);
				OutputStream os = blob.getBinaryOutputStream();
				IOUtils.copy(is, os);
				is.close();
				os.close();
				printQuery(query, data, params);
			}
		} catch (Exception e) {
			printQuery(query, data, params, e);
			throw e;
		} finally {
			try {
				rs.close();
			} catch (Exception e) {
			}
			try {
				pstmt.close();
			} catch (Exception e) {
			}
		}
	}

	/**
	 * BLOB ͸ ȸؼ OutputStream  Ѵ.
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @param colunmName
	 * @param os
	 * @throws Exception
	 */
	protected void executeQueryBlob(String query, Data data, String[] params,
			String colunmName, OutputStream os) throws Exception {
		PreparedStatement pstmt = null;
		ResultSet rs = null;

		try {
			pstmt = conn.prepareStatement(query);
			this.setParams(pstmt, data, params);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				BLOB blob = (BLOB) rs.getBlob(colunmName);
				InputStream is = blob.getBinaryStream();
				IOUtils.copy(is, os);
				is.close();
				os.close();
			}
			printQuery(query, data, params);
		} catch (Exception e) {
			printQuery(query, data, params,e);
			throw e;
		} finally {
			try {
				rs.close();
			} catch (Exception e) {
			}
			try {
				pstmt.close();
			} catch (Exception e) {
			}
		}
	}

	/**
	 *  "?" ڸ   ġ  
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @return
	 */
	protected String makeQuery(String query, Data data, String[] params) {

		for (int i = 0; i < params.length; i++) {
			query = query.replaceFirst("\\?", "'" + data.getString(params[i])
					+ "'");
		}
		return query;
	}

	/**
	 *  "?" ڸ   ġ  
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @return
	 */
	protected String makeQueryProc(String query, Data data, String[] params) {

		for (int i = 0; i < params.length; i++) {
			query = query.replaceFirst("\\?", "'"
					+ data.getString(params[i].replaceAll("I:", "")) + "'");
		}
		return query;
	}

	/**
	 * ν  <br>
	 * params 迭  Է½ <br>
	 * IN ̸ "I:parameter" <br>
	 * OUT ̸ "O:VARCHAR" <br>
	 *  ó Է Ѵ. OUT ´ ⺻ varchar  ⺻ Ѵ.
	 * 
	 * @param query
	 * @param data
	 * @param params
	 * @param colunmName
	 * @param content
	 * @throws Exception
	 */
	protected String[] executeProc(String query, Data data, String[] params)
			throws Exception {
		printQuery(makeQueryProc(query, data, params), data, params);

		CallableStatement cstmt = null;
		Vector v = new Vector();
		String[] nRetArr = null;
		try {
			// Proc IN 
			cstmt = conn.prepareCall(query);
			String[] nParams = new String[params.length];
			for (int i = 0; i < params.length; i++) {
				String vType = (params[i].trim().split(":"))[0];// Ÿ(IN,OUT)
				String vName = (params[i].trim().split(":"))[1];// 
				if (vType.equals("I")) {
					cstmt.setString(i + 1, data.getString(vName.trim()));
					nParams[i] = vName.trim();
				} else if (vType.equals("O")) {
					cstmt.registerOutParameter(i + 1, java.sql.Types.VARCHAR);
					v.add(new Integer(i + 1));
					nParams[i] = "OUT";
				}
			}

			cstmt.execute();

			// Proc OUT 
			nRetArr = new String[v.size()];

			for (int j = 0; j < v.size(); j++) {
				nRetArr[j] = cstmt.getString(new Integer(v.get(j).toString())
						.intValue());
			}
			
			printQuery(makeQueryProc(query, data, params), new Data(), new String[0]);
		} catch (SQLException e) {
			printQuery(makeQueryProc(query, data, params), new Data(), new String[0], e);
			throw e;
		} finally {
			try {
				cstmt.close();
			} catch (Exception e) {
			}
		}
		return nRetArr;
	}

	private void printQuery(String query, Data data, String[] params) {
		boolean b = Boolean.parseBoolean(CommonMap
				.getValue("/prj/system/debugPrint/sql"));
		if (!b)
			return;

		java.util.Calendar now = java.util.Calendar.getInstance();
		java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat(
				"yyyyMMdd");
		String filename = CommonMap.getValue("/prj/system/debugPrint/sqlPath")
				+ "/" + sdf.format(now.getTime());
		java.io.File file = new java.io.File(filename + ".log");
		if (!file.exists())
			try {
				file.createNewFile();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		String str = makeQuery(query, data, params);
		try {
			FileWriter writer = new FileWriter(file, true);
			writer.write("===================== " + this.getClass().getName()
					+ " =====================\n");
			writer.write(str);
			writer
					.write("\n=================================================================================\n");
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void printQuery(String query, Data data, String[] params,
			Exception e) {
		boolean b = Boolean.parseBoolean(CommonMap
				.getValue("/prj/system/debugPrint/sql"));
		if (!b)
			return;

		java.util.Calendar now = java.util.Calendar.getInstance();
		java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat(
				"yyyyMMdd");
		String filename = CommonMap.getValue("/prj/system/debugPrint/sqlPath")
				+ "/" + sdf.format(now.getTime());
		java.io.File file = new java.io.File(filename + ".log");
		if (!file.exists())
			try {
				file.createNewFile();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		String str = makeQuery(query, data, params);
		try {
			FileWriter writer = new FileWriter(file, true);
			writer.write("===================== " + this.getClass().getName()
					+ " =====================\n");
			writer.write(str);
			
			writer.write("\n\n");
			StackTraceElement ste[] = e.getStackTrace();
			for (int i = 0; i < ste.length; i++)
				writer.write(ste[i].toString()+"\n");
			writer
					.write("\n=================================================================================\n");
			writer.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
}
