package com.lsf.dao;

import java.sql.Connection;
import java.sql.SQLException;

import com.lsf.db.DBConnection;
import com.lsf.support.Data;
import com.lsf.support.MultiData;

/**
 * AbstractDao  ӹ޾ ó Ѵ.
 *  Ŭ ӹ ڿ Connection  ѱ ʰ Connection Ŭ  Ѵ.
 * Transaction ó ϰ Ѵٸ execute ޼带 ϱ  startTransaction 
 * execute ޼带 ϰ  endTransaction  ȣϸ ȴ.
 * startTransaction  endTransaction   쿡 ޼ ȣ ܵ Transaction 
 * óѴ.
 *
 * @author mcheonho
 *
 */
public abstract class AbstractTransDao extends AbstractDao {

	public AbstractTransDao() {
		this(null);
	}

	private AbstractTransDao(Connection conn) {
		super(null);
	}

	protected MultiData executeQueryList(String query, Data data, String[] params, int page, int pageSize) throws SQLException {
		boolean localTransaction = false;
		if (conn == null) {
			startTransaction();
			localTransaction = true;
		}

		MultiData mData = super.executeQueryList(query, data, params, page, pageSize);

		if (localTransaction) {
			endTransaction();
			conn = null;
		}

		return mData;
	}

	protected MultiData executeQueryList(String query, Data data, String[] params) throws SQLException {
		boolean localTransaction = false;
		if (conn == null) {
			startTransaction();
			localTransaction = true;
		}

		MultiData mData = super.executeQueryList(query, data, params);

		if (localTransaction) {
			endTransaction();
			conn = null;
		}

		return mData;
	}

	protected Data executeQueryListWithHeader(String query, Data data, String[] params, int start, int pageSize) throws SQLException {
		boolean localTransaction = false;
		if (conn == null) {
			startTransaction();
			localTransaction = true;
		}

		Data result = super.executeQueryListWithHeader(query, data, params, start, pageSize);

		if (localTransaction) {
			endTransaction();
			conn = null;
		}

		return result;
	}

	protected Data executeQuerySingle(String query, Data data, String[] params) throws SQLException {
		boolean localTransaction = false;
		if (conn == null) {
			startTransaction();
			localTransaction = true;
		}

		Data result = super.executeQuerySingle(query, data, params);

		if (localTransaction) {
			endTransaction();
			conn = null;
		}

		return result;
	}

	protected int executeUpdate(String query, Data data, String[] params) throws SQLException {
		boolean localTransaction = false;
		if (conn == null) {
			startTransaction();
			localTransaction = true;
		}

		int i = super.executeUpdate(query, data, params);

		if (localTransaction) {
			endTransaction();
			conn = null;
		}

		return i;
	}

	protected int executeUpdate(String query) throws SQLException {
		boolean localTransaction = false;
		if (conn == null) {
			startTransaction();
			localTransaction = true;
		}

		int i = super.executeUpdate(query);

		if (localTransaction) {
			endTransaction();
			conn = null;
		}

		return i;
	}

	protected void startTransaction() throws SQLException {
		this.conn = DBConnection.getConnection();
		conn.setAutoCommit(false);
	}

	public void endTransaction() throws SQLException {
		conn.commit();
		DBConnection.close(conn);
	}
}
