/**
 * Copyright(c) 2001 iSavvix Corporation (http://www.isavvix.com/)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of iSavvix Corporation not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * iSavvix Corporation.
 *
 * ISAVVIX CORPORATION MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL ISAVVIX CORPORATION OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 *
 */

package com.lsf.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.pool.ObjectPool;
import org.apache.commons.pool.impl.GenericObjectPool;

import com.lsf.util.CommonUtil;

public class DBConnectionMgrJDBC {

	private HashMap dsMap;

	private static DBConnectionMgrJDBC me;

	private DBConnectionMgrJDBC() {
		dsMap = new HashMap();
	}

	private synchronized static DBConnectionMgrJDBC getInstance() {
		if (me == null) {
			me = new DBConnectionMgrJDBC();
		}
		return me;
	}

	private static PoolingDataSource setupDataSource(String name) {
		PoolingDataSource ds = (PoolingDataSource) getInstance().dsMap.get(name);

		if (ds == null) {

			if (DBConf.getValue(name + "/driver") == null)
				return null;

			String dbDriver = DBConf.getValue(name + "/driver");
			String dbUrl = DBConf.getValue(name + "/jdbcurl");
			String dbUser = DBConf.getValue(name + "/user");
			String dbPass = CommonUtil.nvl(DBConf.getValue(name + "/passwd"));
			int maxConn = NumberUtils.stringToInt(DBConf.getValue(name
					+ "/maxConn"));

			try {
				Class.forName(dbDriver);
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			ObjectPool connectionPool = new GenericObjectPool(null);
	        ConnectionFactory connectionFactory = new DriverManagerConnectionFactory(dbUrl,dbUser,dbPass);
	        PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(connectionFactory,connectionPool,null,null,false,true);
	        ds = new PoolingDataSource(connectionPool);

			getInstance().dsMap.put(name, ds);
		}

		return ds;
	}

	public static Connection getConnection() throws SQLException {
		return getConnection("default");
	}

	public static Connection getConnection(String name) throws SQLException {
		PoolingDataSource ds = setupDataSource(name);
		if (ds == null)
			return null;
		return ds.getConnection();
	}

	public static void closePool(String name) throws SQLException {
		BasicDataSource ds = (BasicDataSource) getInstance().dsMap.get(name);
		if (ds != null)
			ds.close();
	}

}